/*
    SDL - Simple DirectMedia Layer
    Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Sam Lantinga
    slouken@libsdl.org
*/
#include "SDL_config.h"

#ifndef _SDL_os2dive_h
#define _SDL_os2dive_h


// OS2 specific includes
#define INCL_TYPES
#define INCL_WIN
#define INCL_DOSPROCESS
#define INCL_DOSRESOURCES
#define INCL_DOSMISC
#define INCL_DOS
#define INCL_DOSERRORS
#define INCL_GPI

#define INCL_OS2MM
#define INCL_MCIOS2
#define INCL_MMIOOS2
#define INCL_MMIO_CODEC
#include <os2.h>
#include <os2me.h>     // DART stuff and MMIO stuff
#include <mmioos2.h>   // mmioFOURCC
#include <dive.h>
#include <fourcc.h>


/* Hidden "this" pointer for the video functions */
#define _THIS	SDL_VideoDevice *_this

/* Fullscreen modes, bpp defines */
#define OS2DIVE_FSMODELIST_BPP8        0
#define OS2DIVE_FSMODELIST_BPP15       1
#define OS2DIVE_FSMODELIST_BPP16       2
#define OS2DIVE_FSMODELIST_BPP24       3
#define OS2DIVE_FSMODELIST_BPP32       4
#define OS2DIVE_FSMODELIST_SIZE        5

typedef struct PixelFormat_s
{
  unsigned char ucRedMask;
  unsigned char ucGreenMask;
  unsigned char ucBlueMask;
  unsigned char ucAlphaMask;

  unsigned char ucRedPosition;
  unsigned char ucGreenPosition;
  unsigned char ucBluePosition;
  unsigned char ucAlphaPosition;

  unsigned char ucRedAdjust;
  unsigned char ucGreenAdjust;
  unsigned char ucBlueAdjust;
  unsigned char ucAlphaAdjust;

} PixelFormat_t, *PixelFormat_p;

typedef struct os2dive_VideoMode_s
{
  unsigned int uiXResolution;
  unsigned int uiYResolution;
  unsigned int uiBPP;
  unsigned int uiScanLineSize;

  PixelFormat_t PixelFormat;

  struct os2dive_VideoMode_s *pNext;

} os2dive_VideoMode, *os2dive_VideoMode_p;

/* Private display data */
struct SDL_PrivateVideoData
{
  HMTX  hmtxUseDive;                // Mutex semaphore to protect DIVE (only 1 thread will call into DIVE at a time)
  HDIVE hDiveInst;                  // Handle for opened DIVE instance
  ULONG ulDiveImageHandle;            // Handle of Dive Image Buffer

  FOURCC fccSurfaceToUse;
  int    iBPPSurfaceToUse ;
  PixelFormat_t pfSurfaceToUse;

  os2dive_VideoMode     desktopVideoModeDesc;
  os2dive_VideoMode_p   pAvailableDiveVideoModes;
  SDL_Rect          **pListModesResult[OS2DIVE_FSMODELIST_SIZE]; // Allocated memory to return list of modes for os2fslib_ListModes() API

  os2dive_VideoMode   SrcBufferDesc;    // Description of current source image buffer
  char               *pchSrcBuffer;     // The source image buffer itself
  SDL_Surface        *pSDLSurface;      // The SDL surface describing the buffer
  HMTX                hmtxUseSrcBuffer; // Mutex semaphore to manipulate src buffer
  HWND                hwndFrame, hwndClient;  // Window handle of frame and client
  int                 iPMThreadStatus;  // 0: Not running
                                        // 1: Running
                                        // Other: Not running, had an error
  int                 tidPMThread;      // Thread ID of PM Thread
  int                 fInFocus;         // True if we're in focus!
  int                 iSkipWMMOUSEMOVE; // Number of WM_MOUSEMOVE messages to skip!
  int                 iMouseVisible;    //

  PFNWP               pfnOldFrameProc;  // Old window frame procedure
  int                 bProportionalResize; // 0: No proportional resizing
                                           // 1: Do proportional resizing
  ULONG               ulResizingFlag;   // First resizing flag value
};


/* OS/2 specific backdoor function to be able to set FrameControlFlags of */
/* the SDL window before creating it. */
extern DECLSPEC void SDLCALL SDL_OS2DIVE_SetFCFToUse(ULONG ulFCF);
/* OS/2 specific backdoor function to be able to set the parent window of */
/* the SDL window before creating it. */
extern DECLSPEC void SDLCALL SDL_OS2DIVE_SetParentHWNDToUse(HWND hwndParent);

#endif /* _SDL_os2dive_h */
