/*
    SDL - Simple DirectMedia Layer
    Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Sam Lantinga
    slouken@libsdl.org
*/
#include "SDL_config.h"

#ifndef _SDL_os2vman_h
#define _SDL_os2vman_h


// OS2 specific includes
#define INCL_TYPES
#define INCL_DOS
#define INCL_DOSERRORS
#define INCL_DOSPROCESS
#define INCL_WIN
#define INCL_GPI
#include <os2.h>
#define INCL_GRE_DEVICE
#define INCL_GRE_DEVMISC
#include <pmddi.h>
#include "gradd.h"

/* Hidden "this" pointer for the video functions */
#define _THIS	SDL_VideoDevice *_this

/* Fullscreen modes, bpp defines */
#define OS2VMAN_FSMODELIST_BPP8        0
#define OS2VMAN_FSMODELIST_BPP15       1
#define OS2VMAN_FSMODELIST_BPP16       2
#define OS2VMAN_FSMODELIST_BPP24       3
#define OS2VMAN_FSMODELIST_BPP32       4
#define OS2VMAN_FSMODELIST_SIZE        5

typedef struct VMANVideoModeDesc_s
{
  unsigned int  uiModeID; // VMAN mode ID

  unsigned int  uiXResolution;
  unsigned int  uiYResolution;
  unsigned int  uiBPP;
  unsigned int  uiScanLineSize;
  unsigned long fccColorEncoding;

  struct VMANVideoModeDesc_s *pNext;

} VMANVideoModeDesc_t, *VMANVideoModeDesc_p;

/* Private display data */
struct SDL_PrivateVideoData
{
  VMANVideoModeDesc_p pAvailableVideoModes;
  VMANVideoModeDesc_p pDesktopVideoMode;
  SDL_Rect          **pListModesResult[OS2VMAN_FSMODELIST_SIZE]; // Allocated memory to return list of modes for os2fslib_ListModes() API

  VMANVideoModeDesc_t SrcBufferDesc;    // Description of current source image buffer
  char               *pchSrcBuffer;     // The source image buffer itself
  SDL_Surface        *pSDLSurface;      // The SDL surface describing the buffer
  HMTX                hmtxUseSrcBuffer; // Mutex semaphore to manipulate src buffer
  HWND                hwndFrame, hwndClient;  // Window handle of frame and client
  int                 iPMThreadStatus;  // 0: Not running
                                        // 1: Running
                                        // Other: Not running, had an error
  int                 tidPMThread;      // Thread ID of PM Thread
  int                 fInFocus;         // True if we're in focus!
  int                 iSkipWMMOUSEMOVE; // Number of WM_MOUSEMOVE messages to skip!
  int                 iMouseVisible;    //

  PFNWP               pfnOldFrameProc;  // Old window frame procedure
  int                 bIsResizable;
  int                 bProportionalResize; // 0: No proportional resizing
                                           // 1: Do proportional resizing
  ULONG               ulResizingFlag;   // First resizing flag value

  int                 bIsInFSMode;      // if TRUE, then we're in FS mode

  VMANVideoModeDesc_p pFSVideoMode;     // The video mode selected for fullscreen mode

};

/* OS/2 specific backdoor function to be able to set FrameControlFlags of */
/* the SDL window before creating it. */
extern DECLSPEC void SDLCALL SDL_OS2VMAN_SetFCFToUse(ULONG ulFCF);
/* OS/2 specific backdoor function to be able to set the parent window of */
/* the SDL window before creating it. */
extern DECLSPEC void SDLCALL SDL_OS2VMAN_SetParentHWNDToUse(HWND hwndParent);

#endif /* _SDL_os2fslib_h */
